@@ -86,7 +86,6 @@ compare_cache (gchar * text_entry, QofEntity * ent, const QofParam * param)
 	if (text_entry)
 		g_free (text_entry);
 	text_entry = NULL;
-	g_free (check);
 }
 
 /** \brief check each parameter and update backend
@@ -99,7 +98,7 @@ static void
 edit_ok_clicked(GtkWidget G_GNUC_UNUSED *widget, gpointer data)
 {
 	GpeExpenseData *context;
-	GpeCurrency * currency;
+	QofCurrency * currency;
 	GtkTextBuffer *buf;
 	GtkTextIter start, end;
 	QofEntity *ent;
@@ -112,8 +111,8 @@ edit_ok_clicked(GtkWidget G_GNUC_UNUSED *widget, gpointer data)
 	buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (context->text_view));
 	gtk_text_buffer_get_bounds (buf, &start, &end);
 	/* exp_note */
-	compare_cache (gtk_text_buffer_get_text (GTK_TEXT_BUFFER (buf), 
-		&start, &end, FALSE), ent, 
+	compare_cache (g_strdup(gtk_text_buffer_get_text (GTK_TEXT_BUFFER (buf), 
+		&start, &end, FALSE)), ent, 
 		qof_class_get_parameter (GPE_QOF_EXPENSES, EXP_NOTE));
 	/* exp_vendor */
 	compare_cache ( g_strdup (gtk_entry_get_text
@@ -140,7 +139,7 @@ edit_ok_clicked(GtkWidget G_GNUC_UNUSED *widget, gpointer data)
 		ent, qof_class_get_parameter (GPE_QOF_EXPENSES, EXP_CATEGORY));
 	/* exp_currency */
 	mnemonic = gtk_combo_box_get_active_text (context->currency_list);
-	currency = gpe_currency_lookup_name ((QofInstance*)ent, mnemonic);
+	currency = qof_currency_lookup_name ((QofInstance*)ent, mnemonic);
 	/* if currency is not found, preserve the cache version. */
 	if (currency)
 		compare_cache (g_strdup_printf ("%d", currency->pq_code), ent, 
@@ -179,10 +178,10 @@ edit_delete_clicked(GtkWidget G_GNUC_UNUSED *widget, gpointer data)
 static void
 create_currency_list(gpointer G_GNUC_UNUSED key, gpointer value, gpointer user_data)
 {
-	GpeCurrency *curr;
+	QofCurrency *curr;
 	GpeExpenseData *context;
 
-	curr = (GpeCurrency*)value;
+	curr = (QofCurrency*)value;
 	context = (GpeExpenseData*)user_data;
 	gtk_combo_box_insert_text (context->currency_list, curr->pq_code, 
 		curr->mnemonic);
@@ -194,7 +193,7 @@ build_categories_string (GSList * catlist)
 {
 	guint id;
 	GSList * iter;
-	const gchar *cat, *col;
+	const gchar *cat;
 	gchar *s = NULL;
 
 	if (!catlist) { return g_strdup(""); }
@@ -202,29 +201,20 @@ build_categories_string (GSList * catlist)
 	{
 		id = GPOINTER_TO_INT(iter->data);
 		cat = gpe_pim_category_name (id);
-		col = gpe_pim_category_colour (id);
 
 		if (cat)
 		{
 			if (s)
 			{
 				/* append as a comma-separated list */
-				gchar *ns = 
-					col ? g_strdup_printf ("%s, <span underline=\"single\" "\
-					"underline_color=\"%s\">%s</span>", s, col, cat) 
-					: g_strdup_printf ("%s, %s", s, cat);
+				gchar *ns = g_strdup_printf ("%s, %s", s, cat);
 				g_free (s);
 				s = ns;
 			}
 			else
-				s = col ? g_strdup_printf ("<span underline=\"single\" "\
-					"underline_color=\"%s\">%s</span>", col, cat) 
-				: g_strdup (cat);
+				s = g_strdup (cat);
 		}
 	}
-
-	g_slist_free (catlist);
-
 	return s;
 }
 
@@ -269,7 +259,7 @@ static void
 cat_populate (const gchar * cat_name)
 {
 	GSList * cat_list, *match;
-	guint max_list;
+	gint max_list;
 
 	match = NULL;
 	cat_list = gpe_pim_categories_list();
@@ -332,7 +322,7 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 	viewport = gtk_viewport_new(NULL, NULL);
 
 	context->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  
+
 	gtk_table_set_col_spacings (GTK_TABLE(table), gpe_spacing);
 	gtk_table_set_row_spacings (GTK_TABLE(table), gpe_spacing);
 	gtk_box_set_spacing (GTK_BOX(top_vbox), gpe_spacing);
@@ -428,7 +418,7 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 	{
 		gtk_table_attach(GTK_TABLE(table), currency_label, 0, 1, pos, pos+1, 
 					GTK_FILL, GTK_FILL, 0, 0);
-		gpe_currency_foreach(create_currency_list, context);
+		qof_currency_foreach(create_currency_list, context);
 		gtk_table_attach(GTK_TABLE(table), GTK_WIDGET(context->currency_list), 1, 
 					3, pos, pos+1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
 		pos++;
@@ -492,11 +482,11 @@ edit_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 
 	param = qof_class_get_parameter(context->entity->e_type, EXP_CURRENCY);
 	{
-		GpeCurrency * currency;
+		QofCurrency * currency;
 		gint32 check, (*int32_getter)   (QofEntity*, const QofParam*);
 		int32_getter = (gint32 (*)(QofEntity*, const QofParam*)) param->param_getfcn;
 		check = int32_getter(context->entity, param);
-		currency = gpe_currency_lookup ((QofInstance*)context->entity, check);
+		currency = qof_currency_lookup ((QofInstance*)context->entity, check);
 		if (currency)
 			PINFO (" currency=%d mnemonic=%s", check, currency->mnemonic);
 		gtk_combo_box_set_active(context->currency_list, check);
@@ -587,7 +577,7 @@ exp_show_entities(QofEntity *ent, gpointer data)
 	param = qof_class_get_parameter(GPE_QOF_EXPENSES, EXP_DATE);
 	qt = (QofTime*)param->param_getfcn (ent, param);
 	qd = qof_date_from_qtime (qt);
-	date_string = qof_date_print (qd, QOF_DATE_FORMAT_LOCALE);
+	date_string = qof_date_print (qd, QOF_DATE_FORMAT_CE);
 	qof_date_free (qd);
 	param = qof_class_get_parameter(GPE_QOF_EXPENSES, EXP_TYPE);
 	dis_string = param->param_getfcn(ent, param);
@@ -610,13 +600,13 @@ exp_show_entities(QofEntity *ent, gpointer data)
 	}
 	else
 	{
-		GpeCurrency * pqc;
+		QofCurrency * pqc;
 		gint32 curr_code, (*int32_getter) (QofEntity *, const QofParam *);
 		param = qof_class_get_parameter (GPE_QOF_EXPENSES, EXP_CURRENCY);
 		int32_getter = (gint32 (*)(QofEntity *, const QofParam *))
 			param->param_getfcn;
 		curr_code = int32_getter (ent, param);
-		pqc = gpe_currency_lookup ((QofInstance*)ent, curr_code);
+		pqc = qof_currency_lookup ((QofInstance*)ent, curr_code);
 		if (pqc)
 			symbol = g_strdup(pqc->symbol);
 	}
@@ -679,6 +669,15 @@ button_press_event (GtkWidget *widget, GdkEventButton *b, gpointer data)
 	return FALSE;
 }
 
+/** \bug Need to provide a better date change mechanism
+
+Should really use QOF_DATE_FORMAT_LOCALE for the display
+without trying to parse that when supporting changing the date
+itself - instead clicking date should probably raise a
+calendar widget and/or some preference setting for the
+chosen format string.
+
+*/
 static void 
 change_date (GtkCellRendererText G_GNUC_UNUSED *cell, 
 			gchar G_GNUC_UNUSED *path_string, gchar *new_text, gpointer data)
@@ -695,7 +694,7 @@ change_date (GtkCellRendererText G_GNUC_UNUSED *cell,
 	param = qof_class_get_parameter(GPE_QOF_EXPENSES, EXP_DATE);
 	time_setter = (void(*)(QofEntity*, QofTime*))param->param_setfcn;
 	/* convert the string to a QofTime */
-	qd = qof_date_parse (new_text, QOF_DATE_FORMAT_UK);
+	qd = qof_date_parse (new_text, QOF_DATE_FORMAT_CE);
 	qt = qof_date_to_qtime (qd);
 	qof_util_param_edit ((QofInstance*)context->entity, param);
 	if ((time_setter && qof_time_is_valid (qt)))
@@ -710,7 +709,6 @@ change_amount (GtkCellRendererText G_GNUC_UNUSED *cell,
 			   gchar G_GNUC_UNUSED *path_string, gchar *new_text, gpointer data)
 {
 	GpeExpenseData *context;
-	Expense_t *qe;
 	QofNumeric amount;
 	void (*numeric_setter)   (QofEntity*, QofNumeric);
 	void (*i32_setter)       (QofEntity*, gint32);
@@ -719,7 +717,7 @@ change_amount (GtkCellRendererText G_GNUC_UNUSED *cell,
 	gdouble d_amount;
 	gint precision;
 	/* detect local currency */
-	GpeCurrency * c;
+	QofCurrency * c;
 	struct lconv lc;
 	gchar * sy;
 	guint def_pq_code;
@@ -728,7 +726,6 @@ change_amount (GtkCellRendererText G_GNUC_UNUSED *cell,
 	g_return_if_fail(context);
 	ENTER (" ");
 	def_pq_code = -1;
-	qe = (Expense_t*)context->entity;
 	param = qof_class_get_parameter(GPE_QOF_EXPENSES, EXP_CURRENCY);
 	i32_setter = (void(*)(QofEntity*, gint32))param->param_setfcn;
 	/* use locale currency */
@@ -736,7 +733,7 @@ change_amount (GtkCellRendererText G_GNUC_UNUSED *cell,
 	DEBUG (" sym=%s", lc.currency_symbol);
 	sy = g_strdup (lc.int_curr_symbol);
 	sy = g_strstrip (sy);
-	c = gpe_currency_lookup_name ((QofInstance*)context->entity, sy);
+	c = qof_currency_lookup_name ((QofInstance*)context->entity, sy);
 	if (c)
 		def_pq_code = c->pq_code;
 	g_free (sy);
@@ -805,24 +802,31 @@ type_compare (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
 	return result;
 }
 
+/** \bug need to support comparing the real date 
+
+The value in the text field could be wrong or in
+a weird locale format that is impossible to parse.
+
+*/
 static gint
 date_compare (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
  		gpointer G_GNUC_UNUSED user_data)
 {
-	QofDate *qd_a, *qd_b;
+	QofTime *qt_a, *qt_b;
+	QofEntity * ent_a, * ent_b;
+	const QofParam * param;
 	gint result;
-	gchar *str_a, *str_b;
 
+	ent_a = ent_b = NULL;
 	result = 0;
-	gtk_tree_model_get(model, a, COL_EXP_DATE, &str_a, -1);
-	gtk_tree_model_get(model, b, COL_EXP_DATE, &str_b, -1);
-	qd_a = qof_date_parse (str_a, QOF_DATE_FORMAT_UK);
-	qd_b = qof_date_parse (str_b, QOF_DATE_FORMAT_UK);
-	result = qof_date_compare (qd_a, qd_b);
-	qof_date_free (qd_a);
-	qof_date_free (qd_b);
-	g_free(str_a);
-	g_free(str_b);
+	gtk_tree_model_get(model, a, COL_ENTITY, &ent_a, -1);
+	gtk_tree_model_get(model, b, COL_ENTITY, &ent_b, -1);
+	param = qof_class_get_parameter (GPE_QOF_EXPENSES, EXP_DATE);
+	qt_a = param->param_getfcn (ent_a, param);
+	qt_b = param->param_getfcn (ent_b, param);
+	result = qof_time_cmp (qt_a, qt_b);
+	qof_time_free (qt_a);
+	qof_time_free (qt_b);
 	return result;
 }
 
@@ -933,7 +937,7 @@ open_new_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 	const QofParam *param;
 	void (*i32_setter)       (QofEntity*, gint32);
 	/* detect local currency */
-	GpeCurrency * c;
+	QofCurrency * c;
 	struct lconv lc;
 	gchar * sy;
 	guint def_pq_code;
@@ -948,7 +952,7 @@ open_new_expense (GtkWidget G_GNUC_UNUSED *w, GpeExpenseData *context)
 	DEBUG (" sym=%s", lc.currency_symbol);
 	sy = g_strdup (lc.int_curr_symbol);
 	sy = g_strstrip (sy);
-	c = gpe_currency_lookup_name (inst, sy);
+	c = qof_currency_lookup_name (inst, sy);
 	if (c)
 		def_pq_code = c->pq_code;
 	g_free (sy);
