@@ -85,8 +85,6 @@ gpe_expense_close(GpeExpenseData *context)
 {
 	qof_main_free(&context->qof);
 	qof_close();
-	if(context->category_list)
-		g_list_free(context->category_list);
 	g_free(context);
 }
 
@@ -120,7 +118,7 @@ gpe_gui_start(int argc, char *argv[], GpeExpenseData *context)
 			"/", SQLITE_DIR, NULL);
 		gpe_home_exists = g_file_test (context->qof.write_file, G_FILE_TEST_IS_DIR);
 		if (!gpe_home_exists)
-			test = g_mkdir (context->qof.write_file, 0700);		
+			test = g_mkdir (context->qof.write_file, 0700);
 		g_free (context->qof.write_file);
 		context->qof.write_file = g_strconcat (ACCESS_METHOD, 
 			":", g_get_home_dir(), "/", SQLITE_DIR, DEFAULT_FILE, NULL);
@@ -186,6 +184,7 @@ main (int argc, char *argv[])
 	debug_on = FALSE;
 	QOF_OP_INIT
 	input_file = NULL;
+	database = g_strdup("gpe_expenses");
 
 #ifdef ENABLE_NLS
 	setlocale (LC_ALL, "");
@@ -218,6 +217,7 @@ main (int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 	gpe_expense_context = gpe_expense_init();
+	g_return_val_if_fail (gpe_expense_context, 1);
 	while ((optc = poptGetNextOpt (pc)) >= 0)
 	{
 		switch (optc)
@@ -332,6 +332,7 @@ main (int argc, char *argv[])
 		}
 		case qof_op_explain:
 		{
+			qof_mod_database (database, &gpe_expense_context->qof);
 			if(!gpe_expense_context->qof.database) 
 			{ 
 				/* Translators: capitalise only the initial letter of error. */
