@@ -97,12 +97,10 @@ typedef struct {
 	GtkEntry         *attendees_entry; /**< String containing series of names of attendees */
 	GtkComboBox      *edit_type_list;  /**< Edit ExpenseType */
 	GtkComboBox      *set_type_list;   /**< Set ExpenseType */
-	GtkComboBox	     *payment_list;    /**< Selected ExpensePayment */
-	GtkComboBox      *categories;      /**< GPE PIM categories */
-	GtkComboBox	     *currency_list;   /**< unsupported */
-	GList *category_list;   /**< The list of category names for the current database. */
-	gint max_categories;     /**< The number of possible categories. */
-	gint ent_category;       /**< The index of the category of this entity in the category list. */
+	GtkComboBox      *payment_list;    /**< Selected ExpensePayment */
+	GtkWidget        *categories;      /**< GPE PIM category button */
+	GtkWidget        *cat_label;       /**< GPE PIM Category string */
+	GtkComboBox      *currency_list;   /**< unsupported */
 }GpeExpenseData;
 
 /** \brief Shutdown the QOF framework
